unit InstallUnit;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	Gauges, ExtCtrls, StdCtrls;

const
	XaviersTIGCCLocation = 'tigcc.exe';
	TPRBuilderLocation = 'tprbuilder.exe';
	IDELocation = 'Bin\ide.exe';
	HelpLocation = 'Doc\tigcc.chm';
	A68kLocation = 'Bin\a68k.exe';
	PackLocation = 'Bin\pack.exe';
	PStarterLocation = 'Lib\pstarter.o';
	BinLocation = 'Bin\';
	ExamplesLocation = 'Examples\';
	ProjectsLocation = 'Projects\';
	CIncludeLocation = 'Include\C\';
	A68kIncludeLocation = 'Include\ASM\';

	EnvRem = 'REM  Environment for TI-GCC Command-Line Compiler';

	EnumRes: Integer = 1234;

	BlockSize = 1024;
	FileSizeEq = 1024 * 64;

type
	TInstallingFiles = class;
	TInstallingFile = class;

	TInstallForm = class(TForm)
		Label1: TLabel;
		InstallingFileLabel: TLabel;
		CancelButton: TButton;
		Panel1: TPanel;
		ProgressBar: TGauge;
		InstallTimer: TTimer;
		procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
		procedure InstallTimerTimer(Sender: TObject);
		procedure FormShow(Sender: TObject);
		procedure FormClose(Sender: TObject; var Action: TCloseAction);
		procedure FormCreate(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
	private
	public
		Aliases: TStringList;
		Files: TInstallingFiles;
		InstallingFile: Integer;
		ModuleHandle: THandle;
		InstallIDE,
		InstallTIGCC,
		InstallExamples,
		InstallAr,
		InstallA68k,
		InstallPack: Boolean;
		function GetUnAliasedFile(const AliasedFile: string): string;
	end;

	TInstallingFiles = class(TCollection);
	TInstallingFile = class(TCollectionItem)
	public
		Form: TInstallForm;
		Handle: THandle;
		AliasedFile: string;
		DestFile: string;
		Size: Integer;
		FileDate: TDateTime;
		procedure Install;
	end;

var
	InstalledBefore: Boolean = False;
	InstallDir: string;
	ProgramFilesDir: string;
	WinDir: string;
	SysDir: string;
	DosFold: string;
	InstallStart: Boolean = False;
	ProgPath: array [0..MAX_PATH] of Char;
	ProgGroupPath: string;
	SystemNT: Boolean = False;
	NoSystemFolderUse: Boolean = False;

implementation

{$R *.DFM}

uses
	ShlObj, ActiveX,
	UtilsDos;

procedure TInstallingFile.Install;
var
	MemHandle: THandle;
	MemPtr: PByteArray;
	F: file;
	H: THandle;
	SizeDone: Integer;
begin
	InstallStart := True;
	try
		if NoSystemFolderUse and ((UpperCase (WithBackslash (ExtractFilePath (DestFile))) = UpperCase (WithBackslash (SysDir))) or (UpperCase (WithBackslash (ExtractFilePath (DestFile))) = UpperCase (WithBackslash (WinDir)))) then
			DestFile := WithBackslash (InstallDir) + BinLocation + ExtractFileName (InstallDir);
		CreatePath (ExtractFilePath (DestFile));
		MemHandle := LoadResource (Form.ModuleHandle, Handle);
		MemPtr := LockResource (MemHandle);
		AssignFile (F, DestFile);
		Rewrite (F, 1);
		SizeDone := 0;
		while SizeDone < Size - BlockSize + 1 do begin
			BlockWrite (F, MemPtr^ [SizeDone], BlockSize);
			with Form.ProgressBar do
				Progress := Progress + BlockSize;
			Inc (SizeDone, BlockSize);
		end;
		if SizeDone <> Size then begin
			BlockWrite (F, MemPtr^ [SizeDone], Size - SizeDone);
			with Form.ProgressBar do
				Progress := Progress + Size - SizeDone;
		end;
		CloseFile (F);
		H := FileOpen (DestFile, fmOpenWrite or fmShareDenyNone);
		if H <> 0 then begin
			FileSetDate (H, DateTimeToFileDate (FileDate));
			FileClose (H);
		end;
		with Form.ProgressBar do
			Progress := Progress + FileSizeEq;
	except
		Application.MessageBox ('Error installing file.', 'Error', mb_OK or mb_IconError);
	end;
	InstallStart := False;
end;

procedure TInstallForm.FormCloseQuery(Sender: TObject;
	var CanClose: Boolean);
begin
	InstallTimer.Enabled := False;
	CanClose := (ModalResult = mrOK) or (Application.MessageBox
		('Are you sure you want to cancel the installation?  If you cancel now, the program will not be completely installed.',
		'Confirm cancellation', mb_YesNo + mb_IconQuestion) = idYes);
	if not CanClose then
		InstallTimer.Enabled := True;
end;

procedure TInstallForm.InstallTimerTimer(Sender: TObject);
var
	CurrentFile: TInstallingFile;
begin
	if not InstallStart then begin
		if (InstallingFile < Files.Count) then begin
			CurrentFile := TInstallingFile (Files.Items [InstallingFile]);
			InstallingFileLabel.Caption := CurrentFile.DestFile;
			Update;
			CurrentFile.Install;
			Inc (InstallingFile);
		end else
			ModalResult := mrOK;
	end;
end;

procedure TInstallForm.FormShow(Sender: TObject);
var
	ResHandle: THandle;
	MemHandle: THandle;
	MemPtr: PChar;
	S,
	AF: string;
begin
	ModuleHandle := HInstance;
	ResHandle := FindResource (ModuleHandle, MakeIntResource (EnumRes), RT_RCDATA);
	if ResHandle = 0 then
		S := ''
	else begin
		MemHandle := LoadResource (ModuleHandle, ResHandle);
		MemPtr := LockResource (MemHandle);
		S := StrPas (MemPtr);
	end;
	while Pos (';', S) > 0 do begin
		AF := Copy (S, 1, Pos (';', S) - 1);
		if ((Pos (UpperCase (IDELocation), UpperCase (AF)) > 0) and (not InstallIDE)) or
			(((Pos (UpperCase (XaviersTIGCCLocation), UpperCase (AF)) > 0) or (Pos (UpperCase (TPRBuilderLocation), UpperCase (AF)) > 0)) and (not InstallTIGCC)) or
			((Pos (UpperCase (ExamplesLocation), UpperCase (AF)) > 0) and ((not InstallExamples) or ((not InstallIDE) and (Pos ('.TPR', UpperCase (AF)) > 0)))) or
			((Pos ('COMPLETION', UpperCase (AF)) > 0) and (not InstallIDE)) or
			(((Pos (UpperCase (A68kLocation), UpperCase (AF)) > 0) or (Pos (UpperCase (A68kIncludeLocation), UpperCase (AF)) > 0)) and (not InstallA68k)) or
			(((Pos (UpperCase (PackLocation), UpperCase (AF)) > 0) or (Pos (UpperCase (PStarterLocation), UpperCase (AF)) > 0)) and (not InstallPack)) or
			((Pos ('UNINST', UpperCase (AF)) > 0) and NoSystemFolderUse) then begin
			System.Delete (S, 1, Pos (';', S));
			System.Delete (S, 1, Pos (';', S));
			System.Delete (S, 1, Pos (';', S));
			System.Delete (S, 1, Pos (';', S));
			Continue;
		end;
		with TInstallingFile (Files.Add) do begin
			Form := Self;
			AliasedFile := AF;
			DestFile := GetUnAliasedFile (AF);
			if NoSystemFolderUse and ((UpperCase (WithBackslash (ExtractFilePath (DestFile))) = UpperCase (WithBackslash (SysDir))) or (UpperCase (WithBackslash (ExtractFilePath (DestFile))) = UpperCase (WithBackslash (WinDir)))) then
				DestFile := WithBackslash (InstallDir) + BinLocation + ExtractFileName (DestFile);
			System.Delete (S, 1, Pos (';', S));
			try
				FileDate := StrToFloat (Copy (S, 1, Pos (';', S) - 1));
			except
				FileDate := Now;
			end;
			System.Delete (S, 1, Pos (';', S));
			try
				Size := StrToInt (Copy (S, 1, Pos (';', S) - 1));
			except end;
			System.Delete (S, 1, Pos (';', S));
			if FileExists (DestFile) and (((GetFileSize (DestFile) = Size) and (Abs (FileDateToDateTime (FileAge (DestFile)) - FileDate) < 1e-10)) or ((Pos ('FOLDERLINK', UpperCase (AF)) > 0) and (FileDateToDateTime (FileAge (DestFile)) >= FileDate))) then begin
				Free;
			end else begin
				with ProgressBar do
					MaxValue := MaxValue + Size + FileSizeEq;
				try
					Handle := FindResource (ModuleHandle, MakeIntResource (StrToInt (Copy (S, 1, Pos (';', S) - 1))), RT_RCDATA);
				except end;
			end;
			System.Delete (S, 1, Pos (';', S));
		end;
	end;
	InstallTimer.Enabled := True;
end;

procedure TInstallForm.FormClose(Sender: TObject;
	var Action: TCloseAction);
begin
	InstallTimer.Enabled := False;
end;

procedure TInstallForm.FormCreate(Sender: TObject);
begin
	Aliases := TStringList.Create;
	Files := TInstallingFiles.Create (TInstallingFile);
	DecimalSeparator := '.';
end;

procedure TInstallForm.FormDestroy(Sender: TObject);
begin
	Files.Free;
	Aliases.Free;
end;

function TInstallForm.GetUnAliasedFile(const AliasedFile: string): string;
var
	Alias: string;
	P1,
	P2: Integer;
begin
	Result := AliasedFile;
	while Pos ('<', Result) > 0 do begin
		P1 := Pos ('<', Result);
		P2 := Pos ('>', Copy (Result, P1 + 1, Length (Result))) + P1;
		if P2 <= P1 then
			P2 := Length (Result) + 1;
		Alias := Copy (Result, P1 + 1, P2 - P1 - 1);
		if (Length (Result) > P2) and (Result [P2 + 1] = '\') then
			Inc (P2);
		Delete (Result, P1, P2 - P1 + 1);
		Insert (WithBackslash (Aliases.Values [Alias]), Result, P1);
	end;
end;

var
	IDList: PItemIDList;
	Malloc: IMalloc;

initialization
	if SHGetSpecialFolderLocation (0, CSIDL_PROGRAMS, IDList) = NOERROR then begin
		SHGetPathFromIDList (IDList, ProgPath);
		if SHGetMalloc (Malloc) = NOERROR then
			Malloc.Free (IDList);
		Malloc := nil;
	end else
		ProgPath := '';
end.
